--[[
Copyright 2010, 2011 William Matthews. All rights reserved.

See LICENSE for license terms.
]]

local CB = select(2,...)
local P = CB.Private
local Trade = P:NewModule("Trade", "AceEvent-3.0")

local ItemSet = P.ItemSet

local fromItems, toItems, cost

local function updateTrade(event)
	cost = GetPlayerTradeMoney() - GetTargetTradeMoney()
	fromItems = ItemSet.new()
	toItems = ItemSet.new()
	for i = 1,MAX_TRADE_ITEMS do
		local _,_,count = GetTradePlayerItemInfo(i)
		local item = GetTradePlayerItemLink(i)
		if item then fromItems:Inc(P.idOfItem(item),count) end
		_,_,count = GetTradeTargetItemInfo(i)
    item = GetTradeTargetItemLink(i)
		if item then toItems:Inc(P.idOfItem(item),count) end
	end
end

local function doClear(event)
	fromItems = nil
	toItems = nil
	cost = nil
end

local function updateAccept(event,player,target) -- 0/1 values: 1=accepted, 0=not accepted
	local doTrade = (player==1) and (target==1)
	P:DebugPrint({
		player=player,
		target=target,
		fromItems=fromItems,
		toItems=toItems,
		cost=cost,
		doTrade=doTrade,
	})
	if doTrade then
		local item,count = next(toItems)
		local function record()
			CB:RecordTransaction(cost,fromItems,toItems)			
			doClear()
		end
		if item then -- if we're getting at least 1 item, then make sure that the trade actually happens (not inventory full!)
			P:ReceiveCallback(item,count,record)
		else
			record()
		end
	end
end

local function updateClosed(event)
	doClear()
end


function Trade:OnInitialize()
	self:RegisterEvent("PLAYER_TRADE_MONEY",updateTrade)  -- player money
	self:RegisterEvent("TRADE_MONEY_CHANGED",updateTrade) -- target money
	self:RegisterEvent("TRADE_PLAYER_ITEM_CHANGED",updateTrade) -- player item set
	self:RegisterEvent("TRADE_TARGET_ITEM_CHANGED",updateTrade) -- target item set
	self:RegisterEvent("TRADE_ACCEPT_UPDATE",updateAccept)
	self:RegisterEvent("TRADE_CLOSED",updateClosed)
end
